import json
import os

import requests
import requests.exceptions

jyp_token = os.environ["JPY_API_TOKEN"]
jyp_server = os.environ["JUPYTERHUB_API_URL"]

r = requests.get(f"{jyp_server}/user", headers={"Authorization": f"token {jyp_token}"})

try:
    r.raise_for_status()
except requests.exceptions.HTTPError as e:
    print(repr(e))
else:
    print(json.dumps(r.json(), indent=2))



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

# Configuration file for JupyterHub
import os

c = get_config()  # noqa: F821

# We rely on environment variables to configure JupyterHub so that we
# avoid having to rebuild the JupyterHub container every time we change a
# configuration parameter.

# Spawn single-user servers as Docker containers
c.JupyterHub.spawner_class = "dockerspawner.DockerSpawner"

# Spawn containers from this image
c.DockerSpawner.image = os.environ["DOCKER_NOTEBOOK_IMAGE"]

# Connect containers to this Docker network
network_name = os.environ["DOCKER_NETWORK_NAME"]
c.DockerSpawner.use_internal_ip = True
c.DockerSpawner.network_name = network_name

# Explicitly set notebook directory because we'll be mounting a volume to it.
# Most `jupyter/docker-stacks` *-notebook images run the Notebook server as
# user `jovyan`, and set the notebook directory to `/home/jovyan/work`.
# We follow the same convention.
notebook_dir = os.environ.get("DOCKER_NOTEBOOK_DIR", "/home/jovyan/work")
c.DockerSpawner.notebook_dir = notebook_dir

# Mount the real user's Docker volume on the host to the notebook user's
# notebook directory in the container
c.DockerSpawner.volumes = {"jupyterhub-user-{username}": notebook_dir}

# Remove containers once they are stopped
c.DockerSpawner.remove = True

# For debugging arguments passed to spawned containers
c.DockerSpawner.debug = True

# User containers will access hub by container name on the Docker network
c.JupyterHub.hub_ip = "jupyterhub"
c.JupyterHub.hub_port = 8080

# Persist hub data on volume mounted inside container
c.JupyterHub.cookie_secret_file = "/data/jupyterhub_cookie_secret"
c.JupyterHub.db_url = "sqlite:////data/jupyterhub.sqlite"

# Allow all signed-up users to login
c.Authenticator.allow_all = True
c.Authenticator.auto_login = True
c.Authenticator.enable_auth_state = True

print("Configuring JupyterHub...")
c.JupyterHub.authenticator_class = "eodh_jpyauth.auth.EODHAuthenticator"
c.JupyterHub.allow_named_servers = True
c.JupyterHub.template_paths = ["/app/templates"]

print("Configuring EODH Authenticator...")
c.EODHAuthenticator.client_id = os.environ["OAUTH_CLIENT_ID"]
c.EODHAuthenticator.client_secret = os.environ["OAUTH_CLIENT_SECRET"]
c.EODHAuthenticator.authorize_url = os.environ["OAUTH_AUTHORIZE_URL"]
c.EODHAuthenticator.token_url = os.environ["OAUTH_TOKEN_URL"]
c.EODHAuthenticator.oauth_callback_url = os.environ["OAUTH_CALLBACK_URL"]
c.EODHAuthenticator.userdata_url = os.environ["OAUTH_USERDATA_URL"]
c.EODHAuthenticator.username_claim = os.environ.get(
    "OAUTH_USERNAME_CLAIM", "preferred_username"
)
c.EODHAuthenticator.scope = ["openid workspaces"]
c.EODHAuthenticator.enable_logout = True
c.EODHAuthenticator.oauth_logout_url = os.environ["OAUTH_LOGOUT_URL"]
c.EODHAuthenticator.oauth_logout_redirect_uri = os.environ["OAUTH_LOGOUT_REDIRECT_URI"]

print("Configuring DockerSpawner...")
c.DockerSpawner.allowed_images = {
    "EO DataHub": "public.ecr.aws/eodh/eodh-default-notebook",
    "Python 3.12": "quay.io/jupyter/base-notebook:python-3.12",
    "R 4.4": "quay.io/jupyter/r-notebook:r-4.4.2",
}



from oauthenticator.generic import GenericOAuthenticator


def main():
    auth = GenericOAuthenticator()
    print(auth.http_client)
    print(auth.username_key)
    print(auth.extra_params)
    print(auth.tls_verify)


if __name__ == "__main__":
    main()



# from kubespawner.slugs import safe_slug
from kubespawner.spawner import KubeSpawner as KubeSpawner_
from traitlets import Unicode


class KubeSpawner(KubeSpawner_):
    public_url_pattern = Unicode(
        None,
        allow_none=False,
        config=True,
        help="The URL pattern for the public URL",
    )

    async def load_user_options(self):
        """
        Call super method then load workspace from user options. Sets the public URL if
        a pattern is provided.
        """
        await super().load_user_options()
        self.workspace = self.user_options.get("workspace", None)
        if not self.workspace:
            exc = NoWorkspaceSet("No workspace detected in user options.")
            self.log.error("%s", exc)
            raise exc
        self.log.info("Workspace set to '%s'", self.workspace)

        if self.public_url_pattern:
            self.public_url = self._expand_user_properties(self.public_url_pattern)
            self.log.info("Public URL set to '%s'", self.public_url)

    # def get_state(self):
    #     """
    #     Add workspace to the state
    #     """
    #     state = super().get_state()
    #     state["workspace"] = self.workspace
    #     return state

    # def get_env(self):
    #     """
    #     Add workspace to the environment
    #     """
    #     env = super().get_env()
    #     env["WORKSPACE"] = self.workspace
    #     return env

    # def load_state(self, state):
    #     """
    #     Load workspace from state
    #     """
    #     super().load_state(state)
    #     self.workspace = state.get("workspace", None)

    def _options_from_form(self, formdata: dict):
        """
        Add workspace to the user options from the form then call super method and
        return the merged options
        """
        user_options = dict(workspace=formdata.pop("workspace", [None])[0])
        if not user_options["workspace"]:
            self.log.warning("No workspace detected in form data")
            raise NoWorkspaceSet("No workspace detected in form data.")

        # Merge with the user options from the form
        user_options |= super()._options_from_form(formdata)
        return user_options

    # def _expand_user_properties(self, template, slug_scheme=None):
    #     """
    #     Expand user properties in template strings
    #     """
    #     safe_username = safe_slug(self.user.name)
    #     safe_workspace = safe_slug(self.workspace)
    #     servername = f"{safe_username}--{safe_workspace}"
    #     ns = dict(
    #         username=safe_slug(self.user.name),
    #         workspace=safe_slug(self.workspace),
    #         servername=servername,
    #         user_server=servername,
    #     )
    #     for attr_name in ("pod_name", "pvc_name", "namespace", "workspace"):
    #         ns[attr_name] = getattr(self, attr_name, f"{attr_name}_unavailable!")
    #     return template.format(**ns)


class NoWorkspaceSet(Exception):
    pass



import logging
import os

import jwt
from fastapi import FastAPI, HTTPException, Request

logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    handlers=[logging.StreamHandler()],
)

ALLOWED_USERS = os.environ.get("ALLOWED_USERS", "").split(",")

app = FastAPI()


@app.middleware("http")
async def extract_oidc_claims(request: Request, call_next):
    logging.debug("oidc: %s", request.url)
    authorization: str = request.headers.get("Authorization")
    if authorization is None or not authorization.startswith("Bearer "):
        logging.debug("oidc: no or invalid authorization header")
        request.state.token = None
        request.state.claims = {}
    else:
        logging.debug("oidc: found valid authorization header")
        request.state.token = authorization.removeprefix("Bearer ")
        try:
            request.state.claims = jwt.decode(
                request.state.token, options={"verify_signature": False}
            )
            logging.debug("oidc: decoded claims: %s", request.state.claims)
        except jwt.exceptions.DecodeError:
            logging.error("oidc: failed to decode token")
            request.state.claims = {}

    response = await call_next(request)
    return response


@app.get("/api/workspaces")
async def get_workspaces(request: Request):
    logging.info("REQUEST: GET /api/workspaces")

    user = request.state.claims["preferred_username"]
    if user not in ALLOWED_USERS:
        logging.error("user %s forbidden", user)
        raise HTTPException(status_code=403, detail="Forbidden")

    return [
        {
            "name": "workspace1",
        },
        {
            "name": "workspace2",
        },
    ]



from oauthenticator.oauth2 import OAuthenticator, OAuthLogoutHandler
from traitlets import Bool, Unicode


class EODHLogoutHandler(OAuthLogoutHandler):
    """Log a user out by clearing both their JupyterHub login cookie and SSO cookie."""

    async def get(self):
        self.log.info("EODH Logout")
        if self.authenticator.enable_logout:
            await self.default_handle_logout()
            await self.handle_logout()

            redirect_url = self.authenticator.oauth_logout_url
            if self.authenticator.oauth_logout_redirect_uri:
                redirect_url += (
                    f"?redirect_uri={self.authenticator.oauth_logout_redirect_uri}"
                )

            self.redirect(redirect_url)
        else:
            await super().get()


class EODHAuthenticator(OAuthenticator):
    enable_logout = Bool(False, config=True)
    oauth_logout_url = Unicode(config=True)
    oauth_logout_redirect_uri = Unicode(config=True)

    logout_handler = EODHLogoutHandler

    def build_auth_state_dict(self, token_info, user_info):
        """
        Add workspaces claim to auth_state if present in user_info.
        """
        auth_state = super().build_auth_state_dict(token_info, user_info)
        if "workspaces" in user_info:
            auth_state["workspaces"] = user_info["workspaces"]
            self.log.info(
                "Workspaces added to auth_state: %s", auth_state["workspaces"]
            )
        else:
            self.log.warning("No workspaces claim in user token info")
        return auth_state
